-- FUNCTION: DumpRequests.vhd
-- AUTHOR:   Christos Zamantzas
-- CREATED:  24 June 2009
-- VERSION:  7

-- Changes in ver.2: Initial conversion to vhdl.
--                   Removed LPMs: Any_Dump, Connect_and, enable_counter
-- Changes in ver.3: changed logic 
--                    - the ORed dump requests include the CS requests.
--                    - syncronisation happens before aclr and for all dump requests.
-- Changes in ver.4: the DUMP_ON has its "cout" output inversed before is fed to the "en_clk" input.
-- Changes in ver.5: registered the DumpReqA_int & DumpReqB_int signals.
-- Changes in ver.6: clean-up and renaming of signals. No semantic changes.
-- Changes in ver.7: changed logic 
--                    - synchronised the sorting of signals to maskable/unmaskable.
--                    - synchronised the removal of dumps from the unused channels.

-- TODO: remove remaining LPM

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE IEEE.numeric_std.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

LIBRARY work;

ENTITY DumpRequests IS 
   PORT
   (
      Connection     : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      Masking        : IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      Dump_A         : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
      Dump_B         : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
      CSD_UNMASK     : IN  STD_LOGIC;
      CSD_MASK       : IN  STD_LOGIC;
      clk            : IN  STD_LOGIC;
      
      UNMaskable     : OUT STD_LOGIC;
      Maskable       : OUT STD_LOGIC;
      DumpReqA       : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
      DumpReqB       : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
   );
END DumpRequests;

ARCHITECTURE RTL OF DumpRequests IS 

COMPONENT dump_on
   PORT(
         clock  : IN STD_LOGIC;
         clk_en : IN STD_LOGIC;
         aclr   : IN STD_LOGIC;
         cout   : OUT STD_LOGIC;
         q      : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
       );
END COMPONENT;

SIGNAL Dump_ON_U      : STD_LOGIC;
SIGNAL Dump_ON_M      : STD_LOGIC;
SIGNAL Dump_UNMASK    : STD_LOGIC;
SIGNAL Dump_MASK      : STD_LOGIC;

SIGNAL DumpReqA_int   : STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL DumpReqB_int   : STD_LOGIC_VECTOR(7 DOWNTO 0);

SIGNAL MaskingN       : STD_LOGIC_VECTOR(15 DOWNTO 0);

SIGNAL UNMaskable_out : STD_LOGIC_VECTOR(3 DOWNTO 0);
SIGNAL Maskable_out   : STD_LOGIC_VECTOR(3 DOWNTO 0);

SIGNAL UNMASK         : STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL MASK           : STD_LOGIC_VECTOR(15 DOWNTO 0);

BEGIN 

inst1 : dump_on
PORT MAP(
       clock  => clk,
       clk_en => NOT(Dump_ON_M),
       aclr   => Dump_MASK,
       cout   => Dump_ON_M
       );

inst2 : dump_on
PORT MAP(
       clock  => clk,
       clk_en => NOT(Dump_ON_U),
       aclr   => Dump_UNMASK ,
       cout   => Dump_ON_U
       );

PROCESS(clk)
BEGIN
   IF (RISING_EDGE(clk)) THEN
      
      MaskingN <= NOT(Masking);
      
      -- Removing dump requests from not connected channels
      DumpReqA_int <= Connection(7 DOWNTO 0)  AND Dump_A;
      DumpReqB_int <= Connection(15 DOWNTO 8) AND Dump_B;
      
      -- Sorting dump requests to MASKABLE and UNMASKABLE
      UNMASK(7 DOWNTO 0)  <= MaskingN(7 DOWNTO 0)  AND DumpReqA_int;
      UNMASK(15 DOWNTO 8) <= MaskingN(15 DOWNTO 8) AND DumpReqB_int;
       
      MASK(7 DOWNTO 0)    <= Masking(7 DOWNTO 0)   AND DumpReqA_int;
      MASK(15 DOWNTO 8)   <= Masking(15 DOWNTO 8)  AND DumpReqB_int;
      
      -- Generating the output signal
      Dump_UNMASK  <= CSD_UNMASK OR 
                      UNMASK(15) OR 
                      UNMASK(14) OR 
                      UNMASK(13) OR 
                      UNMASK(12) OR 
                      UNMASK(11) OR 
                      UNMASK(10) OR 
                      UNMASK(9)  OR 
                      UNMASK(8)  OR 
                      UNMASK(7)  OR 
                      UNMASK(6)  OR 
                      UNMASK(5)  OR 
                      UNMASK(4)  OR 
                      UNMASK(3)  OR 
                      UNMASK(2)  OR 
                      UNMASK(1)  OR 
                      UNMASK(0);
      
      Dump_MASK    <= CSD_MASK   OR 
                      MASK(15)   OR 
                      MASK(14)   OR 
                      MASK(13)   OR 
                      MASK(12)   OR 
                      MASK(11)   OR 
                      MASK(10)   OR 
                      MASK(9)    OR 
                      MASK(8)    OR 
                      MASK(7)    OR 
                      MASK(6)    OR 
                      MASK(5)    OR 
                      MASK(4)    OR 
                      MASK(3)    OR 
                      MASK(2)    OR 
                      MASK(1)    OR 
                      MASK(0);
      
      -- count-up unless dump request (UNMASKABLE).
      UNMaskable_out <= UNMaskable_out + '1';
      IF Dump_ON_U = '0' THEN 
         UNMaskable_out <= (others => '0');
      END IF;
      
      -- count-up unless dump request (MASKABLE).
      Maskable_out <= Maskable_out + '1';
      IF Dump_ON_M = '0' THEN 
         Maskable_out <= (others => '0');
      END IF;
      
   END IF;
END PROCESS;

UNMaskable  <= UNMaskable_out(3); -- use the MSB of the counter to create the toggling output (UNMASKABLE).
Maskable    <= Maskable_out(3);   -- use the MSB of the counter to create the toggling output (MASKABLE).
DumpReqA    <= DumpReqA_int;
DumpReqB    <= DumpReqB_int;

END RTL;