-- BEAM PERMIT LOGGING CONTROL
   -- Four-State Moore State Machine  
   
-- VERSION: 11
-- DATE: 15/04/2009
-- AUTHOR: Christos Zamantzas

-- changes in ver.07: added read_last_addr signal to notify when last address has been read.
-- changes in ver.08: added one cycle delay in the reading of the memory data.
-- changes in ver.09: merged reading and writing; added two cycles delay in the reading of the memory data.
-- changes in ver.10: removed the "temp_addr" signal (wasn't being used).
-- changes in ver.11: rename of signals. no semantic changes.

-- stats:   1 x M4K  for the FIFO (could be replaced with M512 if depth = 32 (now =256)), 
--          1 x M512 for the memory, 
--        120 x LE   here!. 



library ieee;
use ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

entity BeamPermit_CTRL is

   port(
      MEM_data_out : IN  STD_LOGIC_VECTOR(7 downto 0);
      FIFO_data    : IN  STD_LOGIC_VECTOR(7 downto 0);
      FIFO_addr    : IN  STD_LOGIC_VECTOR(3 downto 0);
      FIFO_empty   : IN  STD_LOGIC;
      VME_rden     : IN  STD_LOGIC;
      VME_addr     : IN  STD_LOGIC_VECTOR(3 downto 0);
      clk          : IN  STD_LOGIC;
      reset        : IN  STD_LOGIC;
      
      MEM_data_in  : OUT STD_LOGIC_VECTOR(7 downto 0);
      MEM_wr_addr  : OUT STD_LOGIC_VECTOR(3 downto 0);
      MEM_wren     : OUT STD_LOGIC;
      MEM_rd_addr  : OUT STD_LOGIC_VECTOR(3 downto 0);
      FIFO_rdreq   : OUT STD_LOGIC   
   );

end entity;

architecture rtl of BeamPermit_CTRL is

   -- Build an enumerated type for the state machine;
   type state_type is (idle, CleanMemoryContents, VMEisReading, RecordingData);

   -- Define Register to hold the current state;
   signal state            :state_type;
   -- Define Registers to hold data or flags;
   signal read_locked      :std_logic;
   signal read_ended       :std_logic;
   signal temp_data_FIFO   :std_logic_vector(7 downto 0);
   signal read_last_addr   :std_logic; 

   signal WordCnt          :integer range 0 to 16;
   
begin

   -- Define Variables;
   
   -- Logic to advance to the next state;
   process (clk, reset)
   begin
      if reset = '1' then
            
         MEM_wren       <= '0'; 
         FIFO_rdreq     <= '0'; 
         WordCnt        <= 0;
         read_last_addr <= '0'; 
         state          <= CleanMemoryContents;
            
      elsif (rising_edge(clk)) then
         MEM_wren    <= '0'; -- dont write if not needed;
            
         case state is
          
            when idle => 
               state <= idle;
               if  (VME_rden ='1') then  
                  state <= VMEisReading;
                  MEM_rd_addr <= VME_addr;
                
               elsif (FIFO_empty = '0') then
                  state       <= RecordingData;
                  WordCnt     <= 0;  -- clear counter; 
                  FIFO_rdreq  <= '1';
                
               end if;
             
            when CleanMemoryContents =>
             
               MEM_data_in <= (others => '0');
               MEM_wr_addr <= std_logic_vector(to_signed((WordCnt),4));
               MEM_wren    <= '1';
               WordCnt     <= WordCnt + 1; -- increment counter;
                  
               state <= CleanMemoryContents; 
                  
               -- end of cleaning;
               if ( WordCnt = 15 ) then
                  state <= idle;
               end if;
             
            when VMEisReading =>
             
               MEM_rd_addr <= VME_addr;
               state <= VMEisReading;
                
               -- reading last address with data;
               if ( VME_addr = "1011" and VME_rden = '1' ) then -- the CPU reads the 12 RS only!!!;
                --if ( VME_addr = "1111" and VME_rden = '1' ) then -- the CPU reads the complete table!!!;
                  read_last_addr <= '1';
                
               end if;
               -- finished reading of data;
               if ( read_last_addr = '1' and VME_rden = '0' ) then
                  WordCnt <= 0;                          -- clear counter for starting the housekeeping;
                  read_last_addr <= '0';
                
                  state <= CleanMemoryContents; 
               end if;
             
            when RecordingData =>
             
               state <= RecordingData;
                
               case WordCnt is
                
                  when 0 =>
                     FIFO_rdreq     <= '0';              -- release flag; new data are now available;
                     temp_data_FIFO <= FIFO_data;        -- store data to use later for writting;
                     MEM_rd_addr    <= FIFO_addr;        -- select read address
                     MEM_wr_addr    <= FIFO_addr;        -- select write address
                  when 1 =>                              -- do nothing; wait for mem data (1st cycle);
                  when 2 =>                              -- do nothing; wait for mem data (2nd cycle);
                  when 3 =>
                     MEM_wren       <= '1';
                     MEM_data_in    <= temp_data_FIFO or MEM_data_out;
                     state          <= idle;
                  when others =>
                     state          <= idle;
                
               end case;
                
               WordCnt <= WordCnt + 1;                   -- increment counter;
             
            when others =>
             
               state <= idle;
          
         end case;
      end if;
   end process;

end rtl;
