---------------------------------------------------------------------------------------------------
-- FILE: CombineData4x.vhd
-- Author: Christos Zamantzas
---------------------------------------------------------------------------------------------------
-- Description:
-- This process will combine the acquired, by the ADC and the counter circuits, data.  
-- In addition a minimum ADC value hold circuit is used to cutoff the noise seen in the ADC value. 

---------------------------------------------------------------------------------------------------
-- Version: 2.4
-- DATE: 06/07/2009

-- ver. 2.1 - First version in VHDL
-- ver. 2.2 - Clean-up of code.
-- ver. 2.3 - Semantic error correction. 
-- ver. 2.4 - Readability changes. 

LIBRARY ieee;
USE ieee.std_logic_1164.all; 

LIBRARY work;

ENTITY CombineData4x IS 
   PORT
   (
      data1x      :  IN  STD_LOGIC_VECTOR(19 DOWNTO 0);
      data2x      :  IN  STD_LOGIC_VECTOR(19 DOWNTO 0);
      data3x      :  IN  STD_LOGIC_VECTOR(19 DOWNTO 0);
      data4x      :  IN  STD_LOGIC_VECTOR(19 DOWNTO 0);
      rden_in     :  IN  STD_LOGIC;
      reset       :  IN  STD_LOGIC;
      clk         :  IN  STD_LOGIC;

      RS01x       :  OUT  STD_LOGIC_VECTOR(19 DOWNTO 0);
      RS02x       :  OUT  STD_LOGIC_VECTOR(19 DOWNTO 0);
      RS03x       :  OUT  STD_LOGIC_VECTOR(19 DOWNTO 0);
      RS04x       :  OUT  STD_LOGIC_VECTOR(19 DOWNTO 0);
      rden_out    :  OUT  STD_LOGIC;
      
      ADC_range1x :  OUT  STD_LOGIC_VECTOR(11 DOWNTO 0);
      ADC_range2x :  OUT  STD_LOGIC_VECTOR(11 DOWNTO 0);
      ADC_range3x :  OUT  STD_LOGIC_VECTOR(11 DOWNTO 0);
      ADC_range4x :  OUT  STD_LOGIC_VECTOR(11 DOWNTO 0)
   );
END CombineData4x;

ARCHITECTURE RTL OF CombineData4x IS 

COMPONENT datacombining
   PORT(
       Din       : IN STD_LOGIC_VECTOR(19 DOWNTO 0);
       rden_in   : IN STD_LOGIC;
       ADC_Range : IN STD_LOGIC_VECTOR(11 DOWNTO 0);
       clk       : IN STD_LOGIC;
       reset     : IN STD_LOGIC;
       Dout      : OUT STD_LOGIC_VECTOR(17 DOWNTO 0);
       rden_out  : OUT STD_LOGIC
       );
END COMPONENT;

COMPONENT range_adc
   PORT(
       Data_in   : IN STD_LOGIC_VECTOR(19 DOWNTO 0);
       rden_in   : IN STD_LOGIC;
       clk       : IN STD_LOGIC;
       Data_out  : OUT STD_LOGIC_VECTOR(19 DOWNTO 0);
       ADC_range : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
       rden_out  : OUT STD_LOGIC
       );
END COMPONENT;

SIGNAL   d_int_1x  :  STD_LOGIC_VECTOR(19 DOWNTO 0);
SIGNAL   d_int_2x  :  STD_LOGIC_VECTOR(19 DOWNTO 0);
SIGNAL   d_int_3x  :  STD_LOGIC_VECTOR(19 DOWNTO 0);
SIGNAL   d_int_4x  :  STD_LOGIC_VECTOR(19 DOWNTO 0);
SIGNAL   en1x      :  STD_LOGIC;
SIGNAL   en2x      :  STD_LOGIC;
SIGNAL   en3x      :  STD_LOGIC;
SIGNAL   en4x      :  STD_LOGIC;
SIGNAL   range1x   :  STD_LOGIC_VECTOR(11 DOWNTO 0);
SIGNAL   range2x   :  STD_LOGIC_VECTOR(11 DOWNTO 0);
SIGNAL   range3x   :  STD_LOGIC_VECTOR(11 DOWNTO 0);
SIGNAL   range4x   :  STD_LOGIC_VECTOR(11 DOWNTO 0);
SIGNAL   RS01x_int :  STD_LOGIC_VECTOR(17 DOWNTO 0);
SIGNAL   RS02x_int :  STD_LOGIC_VECTOR(17 DOWNTO 0);
SIGNAL   RS03x_int :  STD_LOGIC_VECTOR(17 DOWNTO 0);
SIGNAL   RS04x_int :  STD_LOGIC_VECTOR(17 DOWNTO 0);

BEGIN 

inst1 : range_adc
PORT MAP(
       rden_in    => rden_in,
       clk        => clk,
       Data_in    => data1x,
       rden_out   => en1x,
       ADC_range  => range1x,
       Data_out   => d_int_1x);

inst2 : range_adc
PORT MAP(
       rden_in    => rden_in,
       clk        => clk,
       Data_in    => data2x,
       rden_out   => en2x,
       ADC_range  => range2x,
       Data_out   => d_int_2x);

inst3 : range_adc
PORT MAP(
       rden_in    => rden_in,
       clk        => clk,
       Data_in    => data3x,
       rden_out   => en3x,
       ADC_range  => range3x,
       Data_out   => d_int_3x);

inst4 : range_adc
PORT MAP(
       rden_in    => rden_in,
       clk        => clk,
       Data_in    => data4x,
       rden_out   => en4x,
       ADC_range  => range4x,
       Data_out   => d_int_4x);

inst5 : datacombining
PORT MAP(
       rden_in    => en1x,
       clk        => clk,
       reset      => reset,
       ADC_Range  => range1x,
       Din        => d_int_1x,
       Dout       => RS01x_int);

inst6 : datacombining
PORT MAP(
       rden_in    => en2x,
       clk        => clk,
       reset      => reset,
       ADC_Range  => range2x,
       Din        => d_int_2x,
       Dout       => RS02x_int);

inst7 : datacombining
PORT MAP(
       rden_in    => en3x,
       clk        => clk,
       reset      => reset,
       ADC_Range  => range3x,
       Din        => d_int_3x,
       Dout       => RS03x_int);

inst8 : datacombining
PORT MAP(
       rden_in    => en4x,
       clk        => clk,
       reset      => reset,
       ADC_Range  => range4x,
       Din        => d_int_4x,
       rden_out   => rden_out,
       Dout       => RS04x_int);

ADC_range1x <= range1x;
ADC_range2x <= range2x;
ADC_range3x <= range3x;
ADC_range4x <= range4x;

RS01x <= "00" & RS01x_int;
RS02x <= "00" & RS02x_int;
RS03x <= "00" & RS03x_int;
RS04x <= "00" & RS04x_int;

END RTL;